<?php
/**
*
* @package jserrlog
* @copyright (c) 2015 c61 c61@yandex.ru http://c61.su
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/
namespace c61\jserrlog\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{	
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var string */
	protected $phpbb_root_path;
	protected $php_ext;
	
	/**
	* Constructor
	* 
	* @param \phpbb\auth\auth $auth
	* @param \phpbb\config\config $config
	* @param \phpbb\template\template $template
	* @param \phpbb\user $user
	* @param \phpbb\db\driver\driver_interface $db
	* @param string $phpbb_root_path Root path
	* @param string $php_ext
	*/
	public function __construct(\phpbb\auth\auth $auth, \phpbb\config\config $config, \phpbb\template\template $template, \phpbb\user $user, \phpbb\db\driver\driver_interface $db, $phpbb_root_path, $php_ext)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->template = $template;
		$this->user = $user;
		$this->db = $db;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}
	
	/**
	* Assign functions defined in this class to event listeners in the core
	*
	* @return array
	* @static
	* @access public
	*/
	static public function getSubscribedEvents()
	{
		$ret = array();
		$ret['core.user_setup'][] = array('lang_jserrlog',0);
		$ret['core.page_footer_after'][] = array('event_jserrlog',-PHP_INT_MAX);
		$ret['core.adm_page_footer'][] = array('event_jserrlog',-PHP_INT_MAX);
		return $ret;
	}
	
	// ON/OFF
	private $onoff_jserrlog = false;

	/**
	* Events
	*/
	public function lang_jserrlog($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'c61/jserrlog',
			'lang_set' => 'jserrlog',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function event_jserrlog($event) {

		global $phpbb_container, $phpbb_log;

		// ON/OFF
		if ( isset($this->config['jserrlog_onoff_jserrlog']) ) {
			$this->onoff_jserrlog = $this->config['jserrlog_onoff_jserrlog'] ? true : false;
		}

		// Switched OFF ?
		if ( !$this->onoff_jserrlog ) return;

		// rootref context
		$context = $phpbb_container->get('template_context');
		$rootref = &$context->get_root_ref();

		if ( !isset($rootref['JSERRLOG']) ) {

			// Add script

			// Script and Additional info for jserrlog
			$user_type = 'user';
			$user_id = '';
			$user_name = '';
			if ( $this->user->data['user_id'] == ANONYMOUS ) {
				$user_type = 'guest';
			} else if ( $this->user->data['is_bot'] ) {
				$user_type = 'bot';
			} else {
				$user_id = ' id ' . $this->user->data['user_id'];
				if ( isset($rootref["S_USERNAME"]) ) {
					$user_name = ', user name ' . $rootref["S_USERNAME"];
				}
			}

			// Add script to beginning of META
			$ignore = (isset($this->config['jserrlog_ignore']) && !empty($this->config['jserrlog_ignore']) && ($this->config['jserrlog_ignore'] != '[]')) ? ('
jsErrLog.domainIgnore = ' . $this->config['jserrlog_ignore'] . ';') : '';
			$thisonly = (isset($this->config['jserrlog_thisonly']) && !empty($this->config['jserrlog_thisonly']) && ($this->config['jserrlog_thisonly'] != '[]')) ? ('
jsErrLog.domainWhitelist = ' . $this->config['jserrlog_thisonly'] . ';') : '';
			$maxrep = isset($this->config['jserrlog_maxrep']) ? ('
jsErrLog.maxRep = ' . $this->config['jserrlog_maxrep'] . ';') : '';
			$trap = isset($this->config['jserrlog_trap']) ? ('
jsErrLog.trapErrors = ' . ($this->config['jserrlog_trap'] ? 'true' : 'false') . ';') : 'false';
			$console = isset($this->config['jserrlog_console']) ? ('
jsErrLog.logToConsole = ' . ($this->config['jserrlog_console'] ? 'true' : 'false') . ';') : 'false';
			$crossorigin = isset($this->config['jserrlog_crossorigin']) ? ('
jsErrLog.ignoreCrossOriginErrors = ' . ($this->config['jserrlog_crossorigin']  ? 'false' : 'true'). ';') : 'false';
			$rootref["META"] ='
<script type="text/javascript" src="' . $this->phpbb_root_path . 'ext/c61/jserrlog/styles/all/template/js/jserrlog.min.js' . '"></script>
<script type="text/javascript">// <![CDATA[
jsErrLog.url = "' . $this->phpbb_root_path . 'jserrlog?sid=' . $this->user->session_id . '";
jsErrLog.info = "' . $user_type . $user_id . $user_name . '";'
. $ignore . $thisonly . $maxrep . $trap . $console . $crossorigin
. '
// ]]></script>

' . (isset($rootref["META"]) ? $rootref["META"] : '');

		} else if ( request_var('sid','') == $this->user->session_id ) {

			// Log

			$i = request_var('i',0);
			$url = request_var('sn','');
			$file = request_var('fl','');
			$line = request_var('ln',0);
			$col = request_var('cn',0);
			$info = request_var('info','');
			$err = request_var('err','Unknown');

			$log_string = '';
			if ( !empty($err) )  $log_string .= '<br /><br />error message » '.$err;
			if ( !empty($url) )  $log_string .= '<br /><br />at url » '.$url;
			if ( !empty($file) ) $log_string .= '<br /><br />at file » '.$file;
			if ( !empty($line) ) $log_string .= '<br /><br />at line » '.$line;
			if ( !empty($col) )  $log_string .= '<br /><br />at column » '.$col;
			if ( !empty($i) )    $log_string .= '<br /><br />error index » '.$i;
			if ( !empty($info) ) $log_string .= '<br /><br />additional info » '.$info;
			$log_string .= '<br /><br />useragent » '.$rootref["S_USER_BROWSER"];

			$user_id = (empty($this->user->data)) ? ANONYMOUS : $this->user->data['user_id'];
			$user_ip = (empty($this->user->ip)) ? '' : $this->user->ip;

	    	$phpbb_log->add('critical', $user_id, $user_ip, 'LOG_JSERRLOG', time(), array($log_string));

			$this->template->assign_vars(array(
				'JSERRLOG_I'		=> $i,
				'JSERRLOG_LOGGED'	=> str_replace(array('<br /><br />',' »'),array('\n\n',':'),$log_string),
				'JSERRLOG_DEBUG'	=> isset($this->config['jserrlog_onoff_debug']) ? $this->config['jserrlog_onoff_debug'] : 0
			));

		}

	}

}
